@@ -17,15 +17,16 @@ Copyright (C) 2010 Hiroki Ohtani(liris)
     License along with this library; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-"""        
+"""
 
 
 import socket
 from urlparse import urlparse
 import os
+import array
 import struct
 import uuid
-import sha
+import hashlib
 import base64
 import logging
 
@@ -63,6 +64,13 @@ class WebSocketException(Exception):
     """
     pass
 
+class WebSocketConnectionClosedException(WebSocketException):
+    """
+    If remote host closed the connection or some network error happened,
+    this exception will be raised.
+    """
+    pass
+
 default_timeout = None
 traceEnabled = False
 
@@ -96,7 +104,7 @@ def getdefaulttimeout():
 
 def _parse_url(url):
     """
-    parse url and the result is tuple of 
+    parse url and the result is tuple of
     (hostname, port, resource path and the flag of secure mode)
 
     url: url string.
@@ -105,7 +113,6 @@ def _parse_url(url):
         raise ValueError("url is invalid")
 
     scheme, url = url.split(":", 1)
-    url = url.rstrip("/")
 
     parsed = urlparse(url, scheme="http")
     if parsed.hostname:
@@ -132,6 +139,9 @@ def _parse_url(url):
     else:
         resource = "/"
 
+    if parsed.query:
+        resource += "?" + parsed.query
+
     return (hostname, port, resource, is_secure)
 
 def create_connection(url, timeout=None, **options):
@@ -142,10 +152,12 @@ def create_connection(url, timeout=None, **options):
     Passing optional timeout parameter will set the timeout on the socket.
     If no timeout is supplied, the global default timeout setting returned by getdefauttimeout() is used.
     You can customize using 'options'.
-    If you set "headers" dict object, you can set your own custom header.
+    If you set "header" dict object, you can set your own custom header.
 
     >>> conn = create_connection("ws://echo.websocket.org/",
-    ...     headers={"User-Agent": "MyProgram"})
+         ...     header={"User-Agent: MyProgram",
+         ...             "x-custom: header"})
+
 
     timeout: socket timeout time. This value is integer.
              if you set None for this value, it means "use default_timeout value"
@@ -180,7 +192,7 @@ class _SSLSocketWrapper(object):
 
     def recv(self, bufsize):
         return self.ssl.read(bufsize)
-    
+
     def send(self, payload):
         return self.ssl.write(payload)
 
@@ -189,7 +201,7 @@ def _is_bool(*values):
     for v in values:
         if v not in _BOOL_VALUES:
             return False
-    
+
     return True
 
 class ABNF(object):
@@ -198,14 +210,14 @@ class ABNF(object):
     see http://tools.ietf.org/html/rfc5234
     and http://tools.ietf.org/html/rfc6455#section-5.2
     """
-    
+
     # operation code values.
     OPCODE_TEXT   = 0x1
     OPCODE_BINARY = 0x2
     OPCODE_CLOSE  = 0x8
     OPCODE_PING   = 0x9
     OPCODE_PONG   = 0xa
-    
+
     # available operation code value tuple
     OPCODES = (OPCODE_TEXT, OPCODE_BINARY, OPCODE_CLOSE,
                 OPCODE_PING, OPCODE_PONG)
@@ -243,7 +255,7 @@ class ABNF(object):
     def create_frame(data, opcode):
         """
         create frame to send text, binary and other data.
-        
+
         data: data to send. This is string value(byte array).
             if opcode is OPCODE_TEXT and this value is uniocde,
             data value is conveted into unicode string, automatically.
@@ -266,7 +278,7 @@ class ABNF(object):
         length = len(self.data)
         if length >= ABNF.LENGTH_63:
             raise ValueError("data is too long")
-        
+
         frame_header = chr(self.fin << 7
                            | self.rsv1 << 6 | self.rsv2 << 5 | self.rsv3 << 4
                            | self.opcode)
@@ -278,7 +290,7 @@ class ABNF(object):
         else:
             frame_header += chr(self.mask << 7 | 0x7f)
             frame_header += struct.pack("!Q", length)
-        
+
         if not self.mask:
             return frame_header + self.data
         else:
@@ -295,15 +307,14 @@ class ABNF(object):
         mask or unmask data. Just do xor for each byte
 
         mask_key: 4 byte string(byte).
-        
+
         data: data to mask/unmask.
         """
-        _m = map(ord, mask_key)
-        _d = map(ord, data)
-        for i in range(len(_d)):
+        _m = array.array("B", mask_key)
+        _d = array.array("B", data)
+        for i in xrange(len(_d)):
             _d[i] ^= _m[i % 4]
-        s = map(chr, _d)
-        return "".join(s)
+        return _d.tostring()
 
 class WebSocket(object):
     """
@@ -322,8 +333,8 @@ class WebSocket(object):
     >>> ws.recv()
     'Hello, Server'
     >>> ws.close()
-    
-    get_mask_key: a callable to produce new mask keys, see the set_mask_key 
+
+    get_mask_key: a callable to produce new mask keys, see the set_mask_key
       function's docstring for more details
     """
     def __init__(self, get_mask_key = None):
@@ -333,7 +344,7 @@ class WebSocket(object):
         self.connected = False
         self.io_sock = self.sock = socket.socket()
         self.get_mask_key = get_mask_key
-        
+
     def set_mask_key(self, func):
         """
         set function to create musk key. You can custumize mask key generator.
@@ -349,7 +360,7 @@ class WebSocket(object):
     def settimeout(self, timeout):
         """
         Set the timeout to the websocket.
-        
+
         timeout: timeout time(second).
         """
         self.sock.settimeout(timeout)
@@ -359,16 +370,17 @@ class WebSocket(object):
         Get the websocket timeout(second).
         """
         return self.sock.gettimeout()
-    
+
     def connect(self, url, **options):
         """
         Connect to url. url is websocket url scheme. ie. ws://host:port/resource
         You can customize using 'options'.
-        If you set "headers" dict object, you can set your own custom header.
-        
+        If you set "header" dict object, you can set your own custom header.
+
         >>> ws = WebSocket()
         >>> ws.connect("ws://echo.websocket.org/",
-        ...     headers={"User-Agent": "MyProgram"})
+                ...     header={"User-Agent: MyProgram",
+                ...             "x-custom: header"})
 
         timeout: socket timeout time. This value is integer.
                  if you set None for this value,
@@ -398,10 +410,9 @@ class WebSocket(object):
             hostport = "%s:%d" % (host, port)
         headers.append("Host: %s" % hostport)
         headers.append("Origin: %s" % hostport)
-   
+
         key = _create_sec_websocket_key()
         headers.append("Sec-WebSocket-Key: %s" % key)
-        headers.append("Sec-WebSocket-Protocol: chat, superchat")
         headers.append("Sec-WebSocket-Version: %s" % VERSION)
         if "header" in options:
             headers.extend(options["header"])
@@ -427,7 +438,7 @@ class WebSocket(object):
             raise WebSocketException("Invalid WebSocket Header")
 
         self.connected = True
-    
+
     def _validate_header(self, headers, key):
         for k, v in _HEADERS_TO_CHECK.iteritems():
             r = headers.get(k, None)
@@ -441,9 +452,9 @@ class WebSocket(object):
         if not result:
             return False
         result = result.lower()
-        
+
         value = key + "258EAFA5-E914-47DA-95CA-C5AB0DC85B11"
-        hashed = base64.encodestring(sha.sha(value).digest()).strip().lower()
+        hashed = base64.encodestring(hashlib.sha1(value).digest()).strip().lower()
         return hashed == result
 
     def _read_headers(self):
@@ -451,7 +462,7 @@ class WebSocket(object):
         headers = {}
         if traceEnabled:
             logger.debug("--- response header ---")
-            
+
         while True:
             line = self._recv_line()
             if line == "\r\n":
@@ -472,12 +483,12 @@ class WebSocket(object):
 
         if traceEnabled:
             logger.debug("-----------------------")
-        
-        return status, headers    
-    
+
+        return status, headers
+
     def send(self, payload, opcode = ABNF.OPCODE_TEXT):
         """
-        Send the data as string. 
+        Send the data as string.
 
         payload: Payload must be utf-8 string or unicoce,
                   if the opcode is OPCODE_TEXT.
@@ -496,7 +507,7 @@ class WebSocket(object):
     def ping(self, payload = ""):
         """
         send ping data.
-        
+
         payload: data payload to send server.
         """
         self.send(payload, ABNF.OPCODE_PING)
@@ -504,7 +515,7 @@ class WebSocket(object):
     def pong(self, payload):
         """
         send pong data.
-        
+
         payload: data payload to send server.
         """
         self.send(payload, ABNF.OPCODE_PONG)
@@ -521,13 +532,13 @@ class WebSocket(object):
     def recv_data(self):
         """
         Recieve data with operation code.
-        
+
         return  value: tuple of operation code and string(byte array) value.
         """
         while True:
             frame = self.recv_frame()
             if not frame:
-                # handle error: 
+                # handle error:
                 # 'NoneType' object has no attribute 'opcode'
                 raise WebSocketException("Not a valid frame %s" % frame)
             elif frame.opcode in (ABNF.OPCODE_TEXT, ABNF.OPCODE_BINARY):
@@ -545,7 +556,7 @@ class WebSocket(object):
 
         return value: ABNF frame object.
         """
-        header_bytes = self._recv(2)
+        header_bytes = self._recv_strict(2)
         if not header_bytes:
             return None
         b1 = ord(header_bytes[0])
@@ -560,15 +571,15 @@ class WebSocket(object):
 
         length_data = ""
         if length == 0x7e:
-            length_data = self._recv(2)
+            length_data = self._recv_strict(2)
             length = struct.unpack("!H", length_data)[0]
         elif length == 0x7f:
-            length_data = self._recv(8)
+            length_data = self._recv_strict(8)
             length = struct.unpack("!Q", length_data)[0]
 
         mask_key = ""
         if mask:
-            mask_key = self._recv(4)
+            mask_key = self._recv_strict(4)
         data = self._recv_strict(length)
         if traceEnabled:
             recieved = header_bytes + length_data + mask_key + data
@@ -576,14 +587,14 @@ class WebSocket(object):
 
         if mask:
             data = ABNF.mask(mask_key, data)
-        
+
         frame = ABNF(fin, rsv1, rsv2, rsv3, opcode, mask, data)
         return frame
 
     def send_close(self, status = STATUS_NORMAL, reason = ""):
         """
         send close data to the server.
-        
+
         status: status code to send. see STATUS_XXX.
 
         reason: the reason to close. This must be string.
@@ -591,7 +602,7 @@ class WebSocket(object):
         if status < 0 or status >= ABNF.LENGTH_16:
             raise ValueError("code is invalid range")
         self.send(struct.pack('!H', status) + reason, ABNF.OPCODE_CLOSE)
-        
+
 
 
     def close(self, status = STATUS_NORMAL, reason = ""):
@@ -626,9 +637,11 @@ class WebSocket(object):
         self.connected = False
         self.sock.close()
         self.io_sock = self.sock
-        
+
     def _recv(self, bufsize):
         bytes = self.io_sock.recv(bufsize)
+        if not bytes:
+            raise WebSocketConnectionClosedException()
         return bytes
 
     def _recv_strict(self, bufsize):
@@ -637,7 +650,7 @@ class WebSocket(object):
         while remaining:
             bytes += self._recv(remaining)
             remaining = bufsize - len(bytes)
-            
+
         return bytes
 
     def _recv_line(self):
@@ -648,21 +661,21 @@ class WebSocket(object):
             if c == "\n":
                 break
         return "".join(line)
-            
+
 class WebSocketApp(object):
     """
-    Higher level of APIs are provided. 
+    Higher level of APIs are provided.
     The interface is like JavaScript WebSocket object.
     """
     def __init__(self, url,
-                 on_open = None, on_message = None, on_error = None, 
+                 on_open = None, on_message = None, on_error = None,
                  on_close = None, keep_running = True, get_mask_key = None):
         """
         url: websocket url.
         on_open: callable object which is called at opening websocket.
           this function has one argument. The arugment is this class object.
         on_message: callbale object which is called when recieved data.
-         on_message has 2 arguments. 
+         on_message has 2 arguments.
          The 1st arugment is this class object.
          The passing 2nd arugment is utf-8 string which we get from the server.
        on_error: callable object which is called when we get error.
@@ -689,7 +702,8 @@ class WebSocketApp(object):
         """
         send message. data must be utf-8 string or unicode.
         """
-        self.sock.send(data)
+        if self.sock.send(data) == 0:
+            raise WebSocketConnectionClosedException()
 
     def close(self):
         """
